# Copyright (c) 2004 Dennis Craven
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import os
import gtk
import gtk.glade
import gtkmozembed
import gtkspell
import gnome
import gobject
import pango
import locale

from datetime import date, datetime
from re import search
from shutil import copy, move
from gettext import gettext as _

from newton import NAME, VERSION, GLADE_DIR, PIXMAPS_DIR, LOCALE_DIR, WIKI_DIR, WIKI_DATA, WIKI_MEDIA, WIKI_STYLES
from NewtonWikiEng import WikiEngine
from NewtonConfig import KEY_WINDOW_POS_X, KEY_WINDOW_POS_Y, KEY_WINDOW_SIZE_X, KEY_WINDOW_SIZE_Y, KEY_HPANED_POS, KEY_IGNORE_CAMELCASE \
        ,KEY_STYLESHEET, KEY_ENABLE_SPELL_CHECK
import NewtonUtils
import NewtonStock 

gtk.glade.bindtextdomain(NAME.lower(), LOCALE_DIR)

class NewtonGui:
    def __init__(self):
        self.gladeXml = gtk.glade.XML(os.path.join(GLADE_DIR, "newton.glade"), "main_window")
        self.prefsXml = gtk.glade.XML(os.path.join(GLADE_DIR, "preferences.glade"))
        self.icon = gtk.gdk.pixbuf_new_from_file(os.path.join(PIXMAPS_DIR, "newton.svg"))

class AppletButton(NewtonGui):
    def __init__(self, config):
        NewtonGui.__init__(self)
        self.config = config
        self.spellcheck_bug = False

        self.topWindow = self.create_top_window()
        self.preferences = self.create_prefs_dialog()
        self.restore_window_state()
        self.state = "browser"

    def create_top_window(self):
        topWindow = self.gladeXml.get_widget("main_window")
        topWindow.set_icon(self.icon)
        topWindow.connect("delete-event", self.on_main_window_delete)
        topWindow.connect("key-press-event", self.on_key_press_event)

        self.browser = BrowserWidget(self.config)
        self.browser.connect("wiki-location-changed", self.on_wiki_location_changed)
        self.browser.connect("wiki-edit-clicked", self.on_wiki_edit_clicked)
        self.browser.connect("wiki-new-clicked", self.on_wiki_new_clicked)
        self.browser.connect("wiki-delete-clicked", self.on_wiki_delete_clicked)
        self.browser.connect("wiki-link-clicked", self.on_wiki_link_clicked)
        self.browser.connect("show", self.browser.on_browser_widget_show)
        self.browser.connect("wiki-link-message", self.on_wiki_link_message)


        self.icons = NewtonStock.IconFactory(self)

        # UI Actions
        window_actions = [
            ('FileMenu', None, _('_File')),
            ('New', gtk.STOCK_NEW, _('New Page'), '<control>N', _('Create a new wiki page'), 'on_new_action'),
            ('Edit', gtk.STOCK_EDIT, _('Edit Page'), '<control>E', _('Edit the current wiki page'), 'on_edit_action'),
            ('Find', gtk.STOCK_FIND, _('Find pattern'), '<control>F', _('Find text in wiki'), 'on_search_button_clicked'),
            ('Delete', gtk.STOCK_DELETE, _('Delete Page'), '<control>D', _('Delete the current wiki page'), 'on_delete_action'),
            ('Rename', None, _('Rename Page'), None, _('Rename the current wiki page'), 'on_rename_action'),
            ('ExportMenu', NewtonStock.STOCK_EXPORT, _('Export to...')),
            ('ExportHTML', NewtonStock.STOCK_WEB, 'HTML', None, _('Export to HTML file'), 'on_export_html_activate'),
            ('EditMenu', None, _('_Edit')),
            ('Save', gtk.STOCK_SAVE, _('Save Page'), '<control>S', _('Save the current wiki page'), 'on_save_action'),
            ('Preferences', gtk.STOCK_PREFERENCES, _('Preferences'), '<control>P', _('Edit preferences'), 'on_preferences_action'),
            ('GoMenu', None, _('_Go')),
            ('Back', gtk.STOCK_GO_BACK, _('Back'), None, _('Go back in history'), 'on_back_action'),
            ('Forward', gtk.STOCK_GO_FORWARD, _('Forward'), None, _('Go forward in history'), 'on_forward_action'),
            ('Home', gtk.STOCK_HOME, _("Home"), '<control>H', _('Go to NewtonHome'), 'on_home_action'),
            ('Refresh', gtk.STOCK_REFRESH, _('Refresh'), '<control>R', _('Refresh current page'), 'on_reload_action'),
            ('HelpMenu', None, _('_Help')),
            ('About', gtk.STOCK_ABOUT, _('About Newton'), None, _('About Newton'), 'on_about_action'),
            ('Bold', gtk.STOCK_BOLD, _('Insert Bold Text'), '<control>B', _('Insert bolded text'), 'on_bold_action'),
            ('Italic', gtk.STOCK_ITALIC, _('Insert Italic Text'), '<control>I', _('Insert italic text'), 'on_italic_action'),
            ('Underline', gtk.STOCK_UNDERLINE, _('Insert Underlined Text'), '<control>U', _('Insert underlined text'), 'on_underline_action'),
            ('Strikethrough', gtk.STOCK_STRIKETHROUGH, _('Insert Strikethrough Text'), '<control>T', _('Insert strikethrough text'), 'on_strikethrough_action'),
            ('WebLink', NewtonStock.STOCK_WEB_LINK, _('Insert Web Link'), None, _('Insert link to a web page'), 'on_web_link_action'),
            ('EmailLink', NewtonStock.STOCK_EMAIL_LINK, _('Insert Email Link'), None, _('Insert link to an email address'), 'on_email_link_action'),
            ('MediaLink', NewtonStock.STOCK_MEDIA_LINK, _('Insert Media Link'), None, _('Insert link to media file'), 'on_media_link_action'),
            ('ImageLink', NewtonStock.STOCK_IMAGE_LINK, _('Insert Image Link'), None, _('Embed local or remote image'), 'on_image_link_action'),
            ('Date', NewtonStock.STOCK_DATE, _('Insert Date'), None, _('Insert the current date'), 'on_date_action'),
            ('Time', NewtonStock.STOCK_TIME, _('Insert Time'), None, _('Insert the current time'), 'on_time_action')
        ]

        self.ag = gtk.ActionGroup('NewtonActions')
        actions = self.fix_actions(window_actions, self)
        self.ag.add_actions(actions)
        self.ui = gtk.UIManager()
        self.ui.insert_action_group(self.ag, 0)
        self.ui.add_ui_from_file(os.path.join(GLADE_DIR, 'newtonui.xml'))
        topWindow.add_accel_group(self.ui.get_accel_group())

        self.menudock = self.gladeXml.get_widget("bonobodockitem1")
        self.tooldock = self.gladeXml.get_widget("bonobodockitem2")
        self.menubar = self.ui.get_widget('/Menubar')
        self.toolbar = self.ui.get_widget('/Toolbar')
        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
        self.menudock.add(self.menubar)
        self.tooldock.add(self.toolbar)

        tree_popup = self.ui.get_widget('/TreePopup')
        self.treeview = WikiTreeView(tree_popup)
        self.treeview.connect("cursor-changed", self.on_tree_cursor_changed)

        # Create and add the search button and entry.
        self.search_widget = gtk.ToolItem()
        hbox = gtk.HBox(False, 3)
        self.search_button = gtk.Button(stock=gtk.STOCK_FIND)
        self.search_button.unset_flags(gtk.CAN_FOCUS)
        self.search_button.set_relief(gtk.RELIEF_NONE)
        self.search_entry = gtk.Entry()
        self.search_entry.set_width_chars(10)
        hbox.pack_start(self.search_button)
        hbox.pack_start(self.search_entry)
        self.search_widget.add(hbox)
        self.toolbar.insert(gtk.SeparatorToolItem(), -1)
        self.toolbar.show_all()
        self.toolbar.insert(self.search_widget, -1)
        self.search_button.connect("clicked", self.on_search_button_clicked)
        self.search_entry.connect("activate", self.on_search_widget_activate)
        self.search_widget.show_all()
        self.search_entry.hide()

        self.editor = EditWidget(self.ui, self.config)
        self.editor.connect("wiki-save-clicked", self.on_wiki_save_clicked)
        self.editor.connect("wiki-cancel-clicked", self.on_wiki_cancel_clicked)
        self.editor.connect("spellcheck-bug", self.on_spell_enable_fail)

#        export_button = gtk.MenuToolButton(NewtonStock.STOCK_EXPORT)
#        export_button.set_menu(self.ui.get_widget('/Menubar/File/Export'))
#        self.toolbar.insert(export_button, -1)
#        export_button.show()

        self.statusbar = self.gladeXml.get_widget("appbar1")
        self.style_combobox = None
        
        treeview_scroller = self.gladeXml.get_widget("treeview_scroller")
        self.hpaned = self.gladeXml.get_widget("hpaned1")
        treeview_scroller.add(self.treeview)
        browser_vbox = self.gladeXml.get_widget("browser_vbox")
        browser_vbox.pack_start(self.browser)
        browser_vbox.pack_start(self.editor)
        return topWindow

    def create_prefs_dialog(self):
        prefs = self.prefsXml.get_widget("preferences_dialog")
        prefs.set_resizable(False)

        camelcase_checkbutton = self.prefsXml.get_widget("camelcase_checkbutton")
        camelcase_checkbutton.set_active(self.config.get_bool(KEY_IGNORE_CAMELCASE, False))

        if not self.spellcheck_bug:
            check_vbox = self.prefsXml.get_widget('vbox128')
            spell_checkbutton = gtk.CheckButton('Enable spell checking')
            spell_checkbutton.set_active(self.config.get_bool(KEY_ENABLE_SPELL_CHECK, True))
            check_vbox.pack_start(spell_checkbutton)
            spell_checkbutton.connect("toggled", self.on_spell_checkbutton_toggled)
            check_vbox.show_all()
#        spell_checkbutton = self.prefsXml.get_widget("spell_checkbutton")
#        spell_checkbutton.set_active(self.config.get_bool(KEY_ENABLE_SPELL_CHECK, True))

        if self.style_combobox:
            self.style_combobox.destroy()

        combo_hbox = self.prefsXml.get_widget("combo_hbox")
        self.style_combobox = gtk.combo_box_new_text()
        combo_hbox.pack_start(self.style_combobox)
        styles = os.listdir(WIKI_STYLES)
        for style in styles:
            if style.endswith(".css"):
                self.style_combobox.append_text(style[:style.rfind(".")])
        self.style_combobox.set_active(styles.index(self.config.get_string(KEY_STYLESHEET, 'style-blue') + ".css"))
        self.style_combobox.show_all()

        camelcase_checkbutton.connect("toggled", self.on_camelcase_checkbutton_toggled)
        self.style_combobox.connect("changed", self.on_style_combobox_changed)

        return prefs

    def on_spell_enable_fail(self, str1, str2):
        self.spellcheck_bug = True

    def fix_actions(self, actions, instance):
        "Helper function to map methods to an instance"
        retval = []
        for i in range(len(actions)):
            curr = actions[i]
            if len(curr) > 5:
                curr = list(curr)
                curr[5] = getattr(instance, curr[5])
                curr = tuple(curr)
                
            retval.append(curr)
        return retval

    def on_search_button_clicked(self, button):
        if self.search_entry.get_property('visible'):
            self.search_entry.hide()
        else:
            self.search_entry.show()
            self.search_entry.grab_focus()

    def on_search_widget_activate(self, obj):
        pattern = self.search_entry.get_text()
        self.search_entry.hide()
        self.browser.display_search_results(pattern)

    def on_web_link_action(self, obj):
        self.editor.web_link_insert_clicked()

    def on_email_link_action(self, obj):
        self.editor.email_link_insert_clicked()

    def on_media_link_action(self, obj):
        self.editor.media_link_insert_clicked()

    def on_image_link_action(self, obj):
        self.editor.image_link_insert_clicked()

    def on_bold_action(self, obj):
        self.editor.bold_insert_clicked()

    def on_italic_action(self, obj):
        self.editor.italic_insert_clicked()

    def on_underline_action(self, obj):
        self.editor.underline_insert_clicked()

    def on_strikethrough_action(self, obj):
        self.editor.strikethrough_insert_clicked()

    def on_date_action(self, obj):
        self.editor.insert_current_date()

    def on_time_action(self, obj):
        self.editor.insert_current_time()

    def on_new_action(self, obj):
        self.browser.on_new_button_clicked(None)

    def on_edit_action(self, obj):
        self.browser.on_edit_button_clicked(None)

    def on_save_action(self, obj):
        self.editor.on_save_button_clicked(None)

    def on_delete_action(self, obj):
        self.browser.on_delete_button_clicked(None)

    def on_rename_action(self, obj):
        dialog = Input(None, _("Enter the new page name."), 
                _("The pages that link to this page can be updated automatically."))
        fix_links = gtk.CheckButton('_Fix links from other pages.')
        fix_links.set_active(True)
        hbox = gtk.HBox(False)
        hbox.pack_start(gtk.Label())
        hbox.pack_start(fix_links, False, False, 3)
        dialog.contents.pack_start(hbox)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            input = dialog.get_input()
            current_title = self.treeview.selected_wiki_page()
            self.treeview.rename_current_page(input)
            move(os.path.join(WIKI_DATA, current_title), os.path.join(WIKI_DATA, input))
            self.treeview.select_wiki_page(input)

            if fix_links.get_active():
                for root, dirs, files in os.walk(WIKI_DATA):
                    for file in files:
                        path = os.path.join(root, file)
                        contents = NewtonUtils.get_file_contents(path)
                        contents = contents.replace(current_title, input)
                        NewtonUtils.write_wiki_file(path, contents)

        dialog.hide()


    def on_back_action(self, obj):
        self.browser.on_back_button_clicked(None)

    def on_forward_action(self, obj):
        self.browser.on_forward_button_clicked(None)

    def on_home_action(self, obj):
        self.browser.on_home_button_clicked(None)

    def on_reload_action(self, obj):
        self.browser.on_refresh_button_clicked(None)

    def on_preferences_action(self, obj):
        self.on_wiki_preferences_clicked(None)

    def on_export_html_activate(self, obj):
        self.browser.on_export_html_activate()

    def on_about_action(self, obj):
        """Displays the about box."""
        pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(PIXMAPS_DIR, "newton.svg"))
        pixbuf = pixbuf.scale_simple(120, 120, gtk.gdk.INTERP_BILINEAR)

        gnome.ui.About(NAME,
                        VERSION,
                        ('Copyright 2005 Dennis Craven'),
                        ('A Personal Wiki applet for the GNOME2 Desktop Environment.'),
                        ['Dennis Craven <dcraven@csd.uwo.ca>'],
                        [''],
                        '',
                        pixbuf).show()

    def on_key_press_event(self, obj, event):
        if event.keyval == gtk.keysyms.Escape:
            self.toggle_window_visible()
        return False

    def on_tree_cursor_changed(self, treeview):
        (model, iter) = self.treeview.get_selection().get_selected()
        self.browser.navigate_to_wiki_page(model.get_value(iter, 1))

    def disable_toolbuttons(self):
        self.ag.get_action("Back").set_sensitive(False)
        self.ag.get_action("Forward").set_sensitive(False)
        self.ag.get_action("Home").set_sensitive(False)
        self.ag.get_action("Refresh").set_sensitive(False)
        self.ag.get_action("Find").set_sensitive(False)
        self.search_button.set_sensitive(False)
        if self.search_entry.get_property("visible"):
            self.search_entry.hide()
        self.treeview.set_sensitive(False)

    def enable_toolbuttons(self):
        self.ag.get_action("Back").set_sensitive(self.browser.engine.can_go_back())
        self.ag.get_action("Forward").set_sensitive(self.browser.engine.can_go_forward())
        self.ag.get_action("Home").set_sensitive(True)
        self.ag.get_action("Refresh").set_sensitive(True)
        self.ag.get_action("Find").set_sensitive(True)
        self.search_button.set_sensitive(True)
        self.treeview.set_sensitive(True)

    def on_wiki_new_clicked(self, blah, pagename):
        if pagename not in self.treeview.treeview_pages():
            self.topWindow.set_title(pagename)
            self.browser.hide()
            self.editor.show_all()
            self.state = "editor"
            self.disable_toolbuttons()
            self.editor.set_edit_wiki_text(pagename, "")
        else:
            dialog = Error(None, _("The page %s already exists.") % pagename,
                    _("Please choose another name for the new page or edit the existing instance."))
            dialog.run()
            dialog.hide()
            return

    def on_wiki_link_clicked(self, widget, pagename):
        self.ag.get_action("Save").set_sensitive(False)
        self.treeview.select_wiki_page(pagename)

    def on_wiki_link_message(self, widget, link):
        self.statusbar.pop()
        if link:
            self.statusbar.push(link)

    def on_wiki_edit_clicked(self, blah, str):
        self.ag.get_action("New").set_sensitive(False)
        self.ag.get_action("Edit").set_sensitive(False)
        self.ag.get_action("Save").set_sensitive(True)
        self.ag.get_action("Delete").set_sensitive(False)
        self.ag.get_action("Find").set_sensitive(False)
        self.search_button.set_sensitive(False)
        if self.search_entry.get_property("visible"):
            self.search_entry.hide()
        self.browser.hide()
        self.editor.show_all()
        self.state = "editor"
        self.editor.set_edit_wiki_text(self.topWindow.get_title(), str)
        self.disable_toolbuttons()

    def on_wiki_save_clicked(self, blah, str):
        self.ag.get_action("New").set_sensitive(True)
        self.ag.get_action("Edit").set_sensitive(True)
        self.ag.get_action("Save").set_sensitive(False)
        self.ag.get_action("Delete").set_sensitive(True)
        self.ag.get_action("Find").set_sensitive(True)
        self.search_button.set_sensitive(True)
        current_page = self.topWindow.get_title()
        path = WIKI_DATA + self.topWindow.get_title()
        NewtonUtils.write_wiki_file(path, str)
        self.enable_toolbuttons()
        if current_page not in self.treeview.treeview_pages():
            self.treeview.add_wiki_page(current_page)
        self.editor.hide()
        self.browser.show_all()
        self.state = "browser"
        self.treeview.select_wiki_page(current_page)
        self.browser.display_content(current_page)

    def on_wiki_cancel_clicked(self, blah, str):
        self.ag.get_action("New").set_sensitive(True)
        self.ag.get_action("Edit").set_sensitive(True)
        self.ag.get_action("Save").set_sensitive(False)
        self.ag.get_action("Delete").set_sensitive(True)
        self.ag.get_action("Find").set_sensitive(True)
        self.editor.hide()
        self.browser.show_all()
        self.state = "browser"
        self.enable_toolbuttons()

    def on_wiki_preferences_clicked(self, button):
        self.preferences.run()
        self.preferences.hide()

    def on_style_combobox_changed(self, combo):
        self.config.set_string(KEY_STYLESHEET, combo.get_active_text())
        self.ag.get_action("Refresh").activate()

    def on_camelcase_checkbutton_toggled(self, button):
        self.config.set_bool(KEY_IGNORE_CAMELCASE, button.get_active())
        self.ag.get_action("Refresh").activate()

    def on_spell_checkbutton_toggled(self, button):
        self.config.set_bool(KEY_ENABLE_SPELL_CHECK, button.get_active())
        if not self.editor.enable_spell_check_toggle(button.get_active()):
            self.preferences.emit('spellcheck-fail', None)

    def on_wiki_location_changed(self, blah, location):
        pagename = location[location.rfind("/") + 1:]
        self.topWindow.set_title(pagename)
        self.ag.get_action("Delete").set_sensitive(pagename != "NewtonHome")
        self.ag.get_action("Rename").set_sensitive(pagename != "NewtonHome")
        self.browser.delete_button.set_sensitive(pagename != "NewtonHome")
        self.ag.get_action("Back").set_sensitive(self.browser.engine.can_go_back())
        self.ag.get_action("Forward").set_sensitive(self.browser.engine.can_go_forward())

    def on_main_window_delete(self, widget, event):
        self.save_window_state()
        self.topWindow.hide()
        return True

    def on_wiki_delete_clicked(self, widget, pagename):
        dialog = Confirm(None, _('Are you sure you want to delete page "%s"?') % pagename,
                _("Deleting this page is permanent and can not be undone."))
        if dialog.run() == gtk.RESPONSE_OK:
            self.browser.current_location = "NewtonHome"
            self.treeview.remove_selected_page()
            try:
                os.remove(WIKI_DATA + pagename)
            except OSError:
                print "OSError Exception thrown!"
        else:
            dialog.hide()

    def on_click_event(self, widget, event):
        print "Clicked"

    def on_button_toggled(self, button):
        if button.get_active():
            self.topWindow.show_all()
            self.restore_window_state()
            if self.state == "browser":
                self.editor.hide()
                self.search_entry.hide()
            else:
                self.browser.hide()
                self.search_entry.hide()
        else:
            self.save_window_state()
            self.topWindow.hide()
        return True

    def toggle_window_visible(self):
        if not self.topWindow.get_property('visible'):
            self.topWindow.show_all()
            self.restore_window_state()
            if self.state == "browser":
                self.editor.hide()
                self.search_entry.hide()
            else:
                self.browser.hide()
                self.search_entry.hide()
        else:
            if not self.topWindow.is_active():
                self.topWindow.present()
            else:
                self.save_window_state()
                self.topWindow.hide()
        return True

    def restore_window_state(self):
        self.topWindow.resize(self.config.get_int(KEY_WINDOW_SIZE_X, 750), self.config.get_int(KEY_WINDOW_SIZE_Y, 500))
        self.topWindow.move(self.config.get_int(KEY_WINDOW_POS_X, 200), self.config.get_int(KEY_WINDOW_POS_Y, 120))
        self.hpaned.set_position(self.config.get_int(KEY_HPANED_POS, 150))

    def save_window_state(self):
        (x, y) = self.topWindow.get_size()
        self.config.set_int(KEY_WINDOW_SIZE_X, x)
        self.config.set_int(KEY_WINDOW_SIZE_Y, y)
        (x, y) = self.topWindow.get_position()
        self.config.set_int(KEY_WINDOW_POS_X, x)
        self.config.set_int(KEY_WINDOW_POS_Y, y)
        pos = self.hpaned.get_position()
        self.config.set_int(KEY_HPANED_POS, pos)

    def on_button_press(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            widget.stop_emission("button-press-event")




class BrowserWidget(gtk.VBox):
    def __init__(self, config):
        self.config = config
        gtk.VBox.__init__(self, False, 3)
        self.mozframe = gtk.Frame()
        self.pack_start(self.mozframe, True, True, 3)

        hbox = gtk.HBox(False, 3)

        self.edit_button = gtk.Button(_("_Edit"), gtk.STOCK_EDIT)
        hbox.pack_start(self.edit_button, False, False, 3)
        self.edit_button.connect("clicked", self.on_edit_button_clicked)

        self.delete_button = gtk.Button(_("_Delete"), gtk.STOCK_DELETE)
        hbox.pack_start(self.delete_button, False, False, 3)
        self.delete_button.connect("clicked", self.on_delete_button_clicked)

        self.new_button = gtk.Button(_("_New"), gtk.STOCK_NEW)
        hbox.pack_start(self.new_button, False, False, 3)
        self.new_button.connect("clicked", self.on_new_button_clicked)

        label = gtk.Label("%s<span weight='bold'> %s </span>%s" \
                % (_("Press"), _("escape to hide"), _("this window.")))
        label.set_use_markup(True)
        label.set_justify(gtk.JUSTIFY_RIGHT)
        label.set_alignment(1.0, 0.5)
        hbox.pack_start(label, True, True, 3)

        self.current_location = "NewtonHome"
        self.engine = WikiEngine(config)

        self.moz = gtkmozembed.MozEmbed()
        self.moz.connect("open-uri", self.on_link_clicked)
        self.moz.connect("realize", self.on_moz_realize)
        self.moz.connect("link-message", self.on_moz_link_message)
        self.mozframe.add(self.moz)
        self.pack_start(hbox, False, False, 3)

        self.moz.show()

    # this is really bad, but it seems that for now, once the mozembed
    # widget is hidden, it cannot be shown again... As a workaround for
    # now, I'm creating a new one each time.. I hope the old gets caught
    # in the garbage collector... damn.
    # now I can't use go_back() etc and need to maintain my own history
    def on_browser_widget_show(self, widget):
        self.mozframe.remove(self.moz)
        self.moz = gtkmozembed.MozEmbed()
        self.moz.connect("open-uri", self.on_link_clicked)
        self.moz.connect("realize", self.on_moz_realize)
        self.moz.connect("link-message", self.on_moz_link_message)
        self.mozframe.add(self.moz)
        self.moz.show()

    def on_export_html_activate(self):
        loc_data = self.engine.get_wiki_content(self.current_location)
        css = NewtonUtils.get_file_contents(os.path.join(WIKI_STYLES, self.config.get_string(KEY_STYLESHEET, "style-blue") + ".css"))
        html = loc_data[1].replace("@STYLE@", css)
        dialog = gtk.FileChooserDialog("Export to HTML...", 
                None, 
                gtk.FILE_CHOOSER_ACTION_SAVE,
                (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        title = self.moz.get_title()
        dialog.set_current_name(title)
        result = dialog.run()
        if result == gtk.RESPONSE_OK:
            dialog.hide()
            filename = dialog.get_filename()

            if not filename.endswith(".html"):
                filename = filename + ".html"

            if not NewtonUtils.check_write_permissions(filename):
                Error(None, "Permission Denied", "You don't have permission to write to the specified location.").run()
                dialog.destroy()
                return

            # Does the file already exist?
            if NewtonUtils.check_path_exists(filename):
                if not FileOverwrite(None, filename).run():
                    dialog.destroy()
                    return

            NewtonUtils.write_wiki_file(filename, html)
        else:
            dialog.destroy()

    def display_search_results(self, pattern):
        data = self.engine.get_search_results(pattern)
        self.render_wiki_content(("Search Results: %s" % pattern, data))

    def on_delete_button_clicked(self, button):
        self.emit("wiki-delete-clicked", self.current_location)

    def on_new_button_clicked(self, button):
        dialog = NewPageDialog()
        result = dialog.run()
        if result == gtk.RESPONSE_OK:
            self.emit("wiki-new-clicked", dialog.get_entry())
        dialog.hide()

    def on_moz_realize(self, widget):
        self.emit("wiki-link-clicked", self.current_location)

    def display_content(self, pagename):
        loc_data = self.engine.get_wiki_content(pagename)
        self.render_wiki_content(loc_data)

    def navigate_to_wiki_page(self, pagename):
        loc_data = self.engine.new_location(pagename)
        self.render_wiki_content(loc_data)

    def render_wiki_content(self, loc_data):
        content = loc_data[1]
        self.moz.open_stream('file://' + WIKI_DIR, 'text/html')
        self.moz.append_data(content, long(len(content)))
        self.moz.close_stream()
        self.current_location = loc_data[0][loc_data[0].rfind("/")+1:]
        self.emit("wiki-location-changed", loc_data[0])

    def on_link_clicked(self, mozembed, gpointer):
        link = self.moz.get_link_message()
        pagename = link[link.rfind("/")+1:]
        path = NewtonUtils.get_local_path_from_uri(link)
        if self.current_location.startswith("Search Results:"):
            wiki_text = NewtonUtils.get_file_contents(path)
            target = self.current_location[self.current_location.rfind(":") + 1:].strip()
            self.emit("wiki-link-clicked", pagename)
            if len(target):
                regex = NewtonUtils.build_case_insensitive_regex(target)
                match = search(r"%s" % regex, wiki_text)
                while match:
                    wiki_text = wiki_text.replace(match.group(1), '<span class="search">%s</span>' % match.group(1))
                    match = search(r"%s" % regex, wiki_text)
                loc_data = self.engine.transform_arbitrary_string(wiki_text, pagename)
                self.render_wiki_content(loc_data)
            return True

        if NewtonUtils.is_external_link(link):
            gnome.url_show(link)
            return True
        path = NewtonUtils.get_local_path_from_uri(link)
        if link.find("@") > 0:
            return False
        if NewtonUtils.is_media_link(link):
            if NewtonUtils.check_path_exists(path):
                success = os.system("gnome-open %s" % path)
                return True
            else:
                dialog = Error(None, _('The media file "%s" does not exist.' % pagename),
                        _("It may be either typed incorrectly or it has been deleted since this link was created."))
                dialog.run()
                dialog.hide()
                return True
        if NewtonUtils.check_path_exists(path):
            self.emit("wiki-link-clicked", path[path.rfind('/')+1:])
        else:
            dialog = Confirm(None, _('The link "%s" points to a page that does not yet exist.' % pagename),
                    _("Would you like to create this page now?"))
            if dialog.run() == gtk.RESPONSE_OK:
                self.emit("wiki-new-clicked", link[link.rfind("/")+1:])
            else:
                dialog.hide()
        return True


    def on_moz_link_message(self, mozembed):
        link = mozembed.get_link_message()
        if not link:
            self.emit("wiki-link-message", "")
            return
        if NewtonUtils.is_external_link(link):
            self.emit("wiki-link-message", "External link: %s" % link)
        elif "@" in link:
            self.emit("wiki-link-message", "Email %s" % link.replace("mailto:", ''))
        elif NewtonUtils.is_media_link(link):
            self.emit("wiki-link-message", "Open: %s" % link[link.rfind("/")+1:])
        else:
            if NewtonUtils.check_path_exists(NewtonUtils.get_local_path_from_uri(link)):
                self.emit("wiki-link-message", "Go to %s" % link[link.rfind("/")+1:])
            else:
                self.emit("wiki-link-message", "Create page %s" % link[link.rfind("/")+1:])

    def on_back_button_clicked(self, widget):
        loc_data = self.engine.go_back()
        self.render_wiki_content(loc_data)

    def on_forward_button_clicked(self, widget):
        loc_data = self.engine.go_forward()
        self.render_wiki_content(loc_data)

    def on_home_button_clicked(self, widget):
        if self.current_location == "NewtonHome":
            return
        loc_data = self.engine.new_location("NewtonHome")
        self.render_wiki_content(loc_data)

    def on_refresh_button_clicked(self, widget):
        loc_data = self.engine.get_wiki_content(self.current_location)
        self.render_wiki_content(loc_data)

    def on_edit_button_clicked(self, widget):
        raw_text = self.engine.get_raw_wiki_text(self.current_location)
        self.emit('wiki-edit-clicked', raw_text)

class EditWidget(gtk.VBox, NewtonGui):
    def __init__(self, ui, config):
        self.ui = ui
        self.config = config
        gtk.VBox.__init__(self, False, 3)

        labelbox = gtk.EventBox()
        labelbox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("slate grey"))
        labelbox.set_border_width(3)
        self.editing_label = gtk.Label()
        self.editing_label.set_alignment(0.1, 0.5)
        self.editing_label.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        pagetitle_font = pango.FontDescription("normal 20")
        self.editing_label.modify_font(pagetitle_font)
        labelbox.add(self.editing_label)

        self.pack_start(labelbox, False, False, 3)

        self.edit_tools = self.ui.get_widget('/EditTools')
        self.edit_tools.set_style(gtk.TOOLBAR_ICONS)
        self.pack_start(self.edit_tools, False, False, 3)
        
        frame = gtk.Frame()
        self.pack_start(frame, True, True, 3)

        hbox = gtk.HBox(False, 3)

        self.cancel_button = gtk.Button(_("_Cancel"), gtk.STOCK_CANCEL)
        hbox.pack_start(self.cancel_button, False, False, 3)
        self.cancel_button.connect("clicked", self.on_cancel_button_clicked)

        self.save_button = gtk.Button(_("_Save"), gtk.STOCK_SAVE)
        hbox.pack_start(self.save_button, False, False, 3)
        self.save_button.connect("clicked", self.on_save_button_clicked)

        label = gtk.Label("%s<span weight='bold'> %s </span>%s" \
                % (_("Press"), _("escape to hide"), _("this window.")))
        label.set_use_markup(True)
        label.set_justify(gtk.JUSTIFY_RIGHT)
        label.set_alignment(1.0, 0.5)
        hbox.pack_start(label, True, True, 3)

        self.editor = gtk.TextView()

        if self.config.get_bool(KEY_ENABLE_SPELL_CHECK, True):
            try:
                self.spell = gtkspell.Spell(self.editor)
            except: # avoid gtkspell issues discovered in 0.0.8
                self.config.set_bool(KEY_ENABLE_SPELL_CHECK, False)
                self.emit('spellcheck-bug', None)
        else:             
            try:
                self.spell = gtkspell.Spell(self.editor)
                self.spell.detach()
            except: # avoid gtkspell issues discovered in 0.0.8
                self.emit('spellcheck-bug', None)
                pass

        font = pango.FontDescription("monospace")
        self.editor.modify_font(font)
        self.editor.set_wrap_mode(gtk.WRAP_WORD)
        scroller = gtk.ScrolledWindow()
        scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scroller.add(self.editor)
        frame.add(scroller)
        self.pack_start(hbox, False, False, 3)
        self.show_all()

    def enable_spell_check_toggle(self, enable):
        if enable:
            try:
                self.spell = gtkspell.Spell(self.editor)
#                self.spell.set_language(locale.getlocale()[0])
                return True
            except:
                self.config.set_bool(KEY_ENABLE_SPELL_CHECK, False)
                return False
        else:
            try:
                self.spell = gtkspell.get_from_text_view(self.editor)
                self.spell.detach()
                return True
            except:
                self.config.set_bool(KEY_ENABLE_SPELL_CHECK, False)
                return True

    def web_link_insert_clicked(self):
        visible_text = ''
        url = ''
        dialog = Input(None, _("Enter the visible text (optional)."),
                _("If left blank, the URL itself will be shown."))
                
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        else:
            visible_text = dialog.get_input()
            dialog.hide()
            dialog = Input(None, _("Enter the URL of your link."),
                    _("This is the target web document."), "http://")
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        else:
            url = dialog.get_input()

        if len(visible_text):
            self.insert_text_at_cursor("[[%s|%s]] " % (url, visible_text))
        else:
            self.insert_text_at_cursor("[[%s]] " % url)

    def email_link_insert_clicked(self):
        visible_text = ''
        address = ''
        dialog = Input(None, _("Enter the visible text (optional)."),
                _("If left blank, the email address itself will be shown."))
                
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        else:
            visible_text = dialog.get_input()
            dialog.hide()
            dialog = Input(None, _("Enter the email address."),
                    _("This is the target email address."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        else:
            address = dialog.get_input()

        if len(visible_text):
            self.insert_text_at_cursor("[[%s|%s]] " % (address, visible_text))
        else:
            self.insert_text_at_cursor("[[%s]] " % address)

    def media_link_insert_clicked(self):
        dialog = Input(None, _("Enter the visible text (optional)."),
                _("If left blank, the file path itself will be shown."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        else:
            visible_text = dialog.get_input()
            dialog.hide()
            dialog = gtk.FileChooserDialog("Inserting media...", 
                    None, 
                    gtk.FILE_CHOOSER_ACTION_OPEN,
                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                     gtk.STOCK_SAVE, gtk.RESPONSE_OK))
            result = dialog.run()
            if not result == gtk.RESPONSE_OK:
                dialog.hide()
                return
            filename = dialog.get_filename()
        dialog.hide()
        if len(visible_text):
            self.insert_text_at_cursor("{{%s|%s}} " % (filename, visible_text))
        else:
            self.insert_text_at_cursor("{{%s}} " % filename)

    def image_link_insert_clicked(self):
        dialog = ImageInsertDialog(None, "http://")
        while True:
            response = dialog.run()
            if not response == gtk.RESPONSE_OK:
                dialog.hide()
                return
            if dialog.validate_size():
                break
            error = Error(dialog, _("The size entered is not valid."),
                    _("Width and height entries must be numbers only, and cannot be empty."))
            error.run()
            error.hide()
        size = dialog.get_size()
        location = dialog.get_location()
        position = dialog.get_position()
        dialog.hide()
        if len(location):
            if position == 'center':
                self.insert_text_at_cursor("{{ %s%s }} " % (location, size))
            elif position == 'float-left':
                self.insert_text_at_cursor("{{%s%s }} " % (location, size))
            elif position == 'float-right':
                self.insert_text_at_cursor("{{ %s%s}} " % (location, size))
            else:
                self.insert_text_at_cursor("{{%s%s}} " % (location, size))

    def bold_insert_clicked(self):
        dialog = Input(None, _("Enter text to be bolded."),
                _("Newton will insert the correct syntax to bold your text."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        text = dialog.get_input()
        self.insert_text_at_cursor("**%s** " % text)

    def italic_insert_clicked(self):
        dialog = Input(None, _("Enter text to be italicized."),
                _("Newton will insert the correct syntax to italicize your text."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        text = dialog.get_input()
        self.insert_text_at_cursor("//%s// " % text)

    def underline_insert_clicked(self):
        dialog = Input(None, _("Enter text to be underlined."),
                _("Newton will insert the correct syntax to underline your text."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        text = dialog.get_input()
        self.insert_text_at_cursor("__%s__ " % text)

    def strikethrough_insert_clicked(self):
        dialog = Input(None, _("Enter text to be strikethrough formatted."),
                _("Newton will insert the correct syntax to strikethrough format your text."))
        result = dialog.run()
        if not result == gtk.RESPONSE_OK:
            dialog.hide()
            return
        text = dialog.get_input()
        self.insert_text_at_cursor("<del>%s</del> " % text)

    def insert_current_date(self):
        buffer = self.editor.get_buffer()
        t = date.today()
        buffer.insert_at_cursor(t.strftime('%x') + ' ')

    def insert_current_time(self):
        buffer = self.editor.get_buffer()
        t = datetime.now()
        buffer.insert_at_cursor(t.strftime('%X') + ' ')

    def insert_text_at_cursor(self, text):
        buffer = self.editor.get_buffer()
        buffer.insert_at_cursor(text)

    def set_page_name(self, pagename):
        self.editing_label.set_text("Editing: %s" % pagename)

    def set_edit_wiki_text(self, pagename, text):
        self.set_page_name(pagename)
        text = text[text.find("\n") + 1:] # lose the moddate
        self.editor.get_buffer().set_text(text)
        self.editor.grab_focus()
        self.editor.place_cursor_onscreen()
        self.editor.get_buffer().set_modified(False)

    def on_cancel_button_clicked(self, widget):
        if self.editor.get_buffer().get_modified():
            dialog = Confirm(None, _('Cancel without saving changes?'),
                    _("There have been changes to this document since last save."))
            if dialog.run() == gtk.RESPONSE_OK:
                pass
            else:
                return
        self.emit("wiki-cancel-clicked", None)

    def on_save_button_clicked(self, widget):
        buffer = self.editor.get_buffer()
        wiki_text = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
        wiki_text = "@@@ %s\n%s" % (NewtonUtils.get_datetime(), wiki_text)
        self.emit("wiki-save-clicked", wiki_text)

class WikiTreeView(gtk.TreeView):
    def __init__(self, menu):
        gtk.TreeView.__init__(self)
        self.popup = menu
        self.init_model()
        self.init_view_columns()
        self.set_rules_hint(True)
        self.set_reorderable(False)
        self.set_headers_visible(False)
        self.populate_wiki_tree()
        self.connect("button-press-event", self.on_button_press_event)

    def on_button_press_event(self, object, event):
        """Handle a mouse button key press."""
        if event.button == 3: # right mouse button - show popup menu
            path = self.get_path_at_pos(int(event.x), int(event.y))
            self.set_cursor(path[0])
            self.popup.popup(None, None, None, event.button, event.time)
            return True

    def populate_wiki_tree(self):
        for root, dirs, files in os.walk(WIKI_DATA):
            for file in files:
                self.add_wiki_page(file)

    def treeview_pages(self):
        list = []
        self.store.foreach(self.add_page_to_pylist, list)
        return list

    def rename_current_page(self, new_name):
        (model, iter) = self.get_selection().get_selected()
        model.set_value(iter, 1, new_name)

    def add_page_to_pylist(self, model, path, iter, list):
        list.append(model.get_value(iter, 1))

    def selected_wiki_page(self):
        (model, iter) = self.get_selection().get_selected()
        return model.get_value(iter, 1)

    def select_wiki_page(self, pagename):
        self.store.foreach(self.select_page, pagename)

    def select_page(self, model, path, iter, pagename):
        if pagename == model.get_value(iter, 1):
            self.set_cursor(path)

    def remove_selected_page(self):
        (model, iter) = self.get_selection().get_selected()
        if iter:
            self.store.remove(iter)
        self.select_wiki_page("NewtonHome")

    def init_model(self):
        self.store = gtk.TreeStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING)
        self.set_model(self.store)

    def init_view_columns(self):
        self.col = gtk.TreeViewColumn()
        self.col.set_title(_('WikiPage'))
        render_pixbuf = gtk.CellRendererPixbuf()
        self.col.pack_start(render_pixbuf, expand=False)
        self.col.add_attribute(render_pixbuf, 'pixbuf', 0)
        render_text = gtk.CellRendererText()
        self.col.pack_start(render_text, expand=True)
        self.col.add_attribute(render_text, 'text', 1)
        self.append_column(self.col)

    def add_wiki_page(self, pagename):
        iter = self.store.append(None, [self.render_icon(gtk.STOCK_NEW, gtk.ICON_SIZE_MENU, None), pagename])

class Dialog(gtk.Dialog):
    def __init__(self, parent, title, buttons, default = None):
        gtk.Dialog.__init__(self, title, parent, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR)

        self.set_border_width(6)
        self.vbox.set_spacing(12)
        self.set_resizable(False)

        for stock, response in buttons:
            self.add_button(stock, response)

        if default is not None:
            self.set_default_response(default)
        else:
            self.set_default_response(buttons[-1][1])

    def get_button(self, index):
        buttons = self.action_area.get_children()
        return index < len(buttons) and buttons[index] or None

class Hig(Dialog):
    def __init__(self, parent, pritext, sectext, stockimage, buttons, default = None):
        """GNOME higified version of the Dialog object. Inherit
        from here if possible when you need a new dialog."""
        Dialog.__init__(self, parent, "", buttons, default)

        # hbox separating dialog image and contents
        hbox = gtk.HBox()
        hbox.set_spacing(12)
        hbox.set_border_width(6)
        self.vbox.pack_start(hbox)

        # set up image
        if stockimage is not None:
            image = gtk.Image()
            image.set_from_stock(stockimage, gtk.ICON_SIZE_DIALOG)
            image.set_alignment(0.5, 0)
            hbox.pack_start(image, False, False)

        # set up main content area
        self.contents = gtk.VBox()
        self.contents.set_spacing(10)
        hbox.pack_start(self.contents)

        label = gtk.Label()
        label.set_markup("<span size=\"larger\" weight=\"bold\">" + pritext + "</span>\n\n" + sectext)
        label.set_line_wrap(True)
        label.set_alignment(0, 0)
        label.set_selectable(True)
        self.contents.pack_start(label)

    def run(self):
        self.show_all()
        response = gtk.Dialog.run(self)
        self.destroy()
        return response


class NewPageDialog(gtk.Dialog):
    def __init__(self):
        """Dialog shown when the user is going to add a new page to the
        wiki."""
        gtk.Dialog.__init__(self, _('New Wiki Page'))
        self.hbox = gtk.HBox(False, 8)
        self.table = gtk.Table(3, 2, False)
        self.label1 = gtk.Label(_('New page title'))
        self.entry = gtk.Entry()
        self.image = gtk.Image()
        self.image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)

        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
        self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
        self.set_default_response(gtk.RESPONSE_OK)
        self.entry.set_activates_default(True)

        self.hbox.set_border_width(8)
        self.vbox.pack_start(self.hbox, False, False, 0)
        self.hbox.pack_start(self.image, gtk.SHRINK)
        
        self.table.set_row_spacings(4)
        self.table.set_col_spacings(4)
        self.hbox.pack_start(self.table)

        self.table.attach(self.label1, 0, 1, 0, 1)
        self.table.attach(self.entry, 1, 2, 0 ,1)
        self.label1.set_mnemonic_widget(self.entry)

        self.set_resizable(False)
        self.show_all()

    def get_entry(self):
        """Get the path in the dialog."""
        return self.entry.get_text()

    def set_entry(self, text):
        """Set the path in the dialog."""
        self.entry.set_text(text)
        self.entry.select_region(-1, -1)

class Input(Hig):
    def __init__(self, parent, pritext, sectext, prefix=None):
        """HIG compliant input dialog with text entry."""
        Hig.__init__(
                self, parent, pritext, sectext, gtk.STOCK_DIALOG_QUESTION,
                [ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ],
                [gtk.STOCK_OK, gtk.RESPONSE_OK] ], gtk.RESPONSE_OK
        )

        self.entry = gtk.Entry()
        self.entry.set_activates_default(True)
        self.contents.pack_start(self.entry, False, False, 3)

        if prefix:
            self.entry.set_text(prefix)

    def get_input(self):
        return self.entry.get_text()

    def set_input(self, text):
        self.entry.set_text(text)


class ImageInsertDialog(Dialog):
    def __init__(self, parent, prefix=None):
        Dialog.__init__(self, parent, "Insert an image..",
                [ [gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL],
                [gtk.STOCK_OK, gtk.RESPONSE_OK]], gtk.RESPONSE_OK)

        self.set_has_separator(True)

        entry_label = gtk.Label()
        pritext = "1. Enter the location of the image file."
        sectext = "   If the file is located locally, you can use the Browse button."
        entry_label.set_markup("<span size=\"larger\" weight=\"bold\">" + pritext + "</span>\n" + sectext)
        entry_label.set_line_wrap(True)
        entry_label.set_alignment(0, 0)
        self.vbox.pack_start(entry_label)

        self.entry = gtk.Entry()
        self.entry.set_activates_default(True)
        self.entry.set_text("http://")
        self.entry.select_region(0, -1)

        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_BUTTON)
        label = gtk.Label(_("Browse"))
        hbox = gtk.HBox(False, 3)
        hbox.pack_start(image)
        hbox.pack_start(label)
        browse_button = gtk.Button()
        browse_button.add(hbox)
        browse_button.connect("clicked", self.on_browse_button_clicked)

        hbox2 = gtk.HBox(False, 8)
        hbox2.pack_start(self.entry)
        hbox2.pack_start(browse_button, False, False, 3)
        self.vbox.pack_start(hbox2)

        filter = gtk.FileFilter()
        patterns = ['*.png', '*.jpg', '*.gif', '*.jpeg']
        for pattern in patterns:
            filter.add_pattern(pattern)
        filter.set_name("Image files")

        self.file_dialog = gtk.FileChooserDialog("Choose an image file...", 
                self, 
                gtk.FILE_CHOOSER_ACTION_OPEN,
                (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                 gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        self.file_dialog.add_filter(filter)

        self.vbox.pack_start(gtk.HSeparator())

        position_label = gtk.Label()
        pritext = "2. Choose a layout, or position for the image."
        sectext = "\n   This determines how the image will fit into the document."
        position_label.set_markup("<span size=\"larger\" weight=\"bold\">" + pritext + "</span>" + sectext)
        position_label.set_line_wrap(True)
        position_label.set_alignment(0, 0)
        self.vbox.pack_start(position_label)

        center_vbox = gtk.VBox(False, 3)
        center_pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(PIXMAPS_DIR, "image-center.svg"))
        center_pixbuf = center_pixbuf.scale_simple(120, 120, gtk.gdk.INTERP_BILINEAR)
        c_image = gtk.Image()
        c_image.set_from_pixbuf(center_pixbuf)
        self.center_radio = gtk.RadioButton(None, None, False)
        abox = gtk.HBox()
        abox.pack_start(gtk.Label())
        abox.pack_start(self.center_radio)
        center_vbox.pack_start(c_image)
        center_vbox.pack_start(gtk.Label('Center'))
        center_vbox.pack_start(abox)

        fright_vbox = gtk.VBox(False, 3)
        fright_pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(PIXMAPS_DIR, "float-right.svg"))
        fright_pixbuf = fright_pixbuf.scale_simple(120, 120, gtk.gdk.INTERP_BILINEAR)
        r_image = gtk.Image()
        r_image.set_from_pixbuf(fright_pixbuf)
        self.fright_radio = gtk.RadioButton(self.center_radio)
        bbox = gtk.HBox()
        bbox.pack_start(gtk.Label())
        bbox.pack_start(self.fright_radio)
        fright_vbox.pack_start(r_image)
        fright_vbox.pack_start(gtk.Label('Float Right'))
        fright_vbox.pack_start(bbox)

        fleft_vbox = gtk.VBox(False, 3)
        fleft_pixbuf = gtk.gdk.pixbuf_new_from_file(os.path.join(PIXMAPS_DIR, "float-left.svg"))
        fleft_pixbuf = fleft_pixbuf.scale_simple(120, 120, gtk.gdk.INTERP_BILINEAR)
        l_image = gtk.Image()
        l_image.set_from_pixbuf(fleft_pixbuf)
        self.fleft_radio = gtk.RadioButton(self.center_radio)
        cbox = gtk.HBox()
        cbox.pack_start(gtk.Label())
        cbox.pack_start(self.fleft_radio)
        fleft_vbox.pack_start(l_image)
        fleft_vbox.pack_start(gtk.Label("Float Left"))
        fleft_vbox.pack_start(cbox)

        hbox3 = gtk.HBox(False, 8)
        hbox3.pack_start(fleft_vbox)
        hbox3.pack_start(gtk.VSeparator(), False, False, 6)
        hbox3.pack_start(center_vbox)
        hbox3.pack_start(gtk.VSeparator(), False, False, 6)
        hbox3.pack_start(fright_vbox)
        self.vbox.pack_start(hbox3)

        self.vbox.pack_start(gtk.HSeparator())

        size_label = gtk.Label()
        pritext = "3. Choose the size of your embedded image."
        sectext = "\n   The image itself will not be altered, just how it is displayed."
        size_label.set_markup("<span size=\"larger\" weight=\"bold\">" + pritext + "</span>" + sectext)
        size_label.set_line_wrap(True)
        size_label.set_alignment(0, 0)
        self.vbox.pack_start(size_label)

        actual_vbox = gtk.VBox(False, 3)
        self.actual_label = gtk.Label('Actual Size')
        actual_vbox.pack_start(self.actual_label, True, True, 3)
        self.actual_radio = gtk.RadioButton(None)
        self.actual_radio.connect("toggled", self.on_actual_radio_toggled)
        zbox = gtk.HBox()
        zbox.pack_start(gtk.Label())
        zbox.pack_start(self.actual_radio)
        actual_vbox.pack_start(zbox, False, False, 3)

        scale_vbox = gtk.VBox(False, 3)
        scale_width_hbox = gtk.HBox(False, 0)
        self.scale_width_label = gtk.Label('Width: ')
        scale_width_hbox.pack_start(self.scale_width_label)
        self.scale_width_entry = gtk.Entry()
        self.scale_width_entry.set_width_chars(5)
        scale_width_hbox.pack_start(self.scale_width_entry, False, False, 3)
        scale_vbox.pack_start(scale_width_hbox)
        self.scale_label = gtk.Label('Scaled')
        scale_vbox.pack_start(self.scale_label)
        self.scale_radio = gtk.RadioButton(self.actual_radio)
        self.scale_radio.connect("toggled", self.on_scale_radio_toggled)
        sbox = gtk.HBox()
        sbox.pack_start(gtk.Label())
        sbox.pack_start(self.scale_radio)
        scale_vbox.pack_start(sbox)

        custom_vbox = gtk.VBox(False, 3)
        custom_width_hbox = gtk.HBox(False, 0)
        self.custom_width_label = gtk.Label('Width: ')
        custom_width_hbox.pack_start(self.custom_width_label)
        self.custom_width_entry = gtk.Entry()
        self.custom_width_entry.set_width_chars(5)
        custom_width_hbox.pack_start(self.custom_width_entry, False, False, 3)
        custom_vbox.pack_start(custom_width_hbox, True, True, 3)
        custom_height_hbox = gtk.HBox(False, 0)
        self.custom_height_label = gtk.Label('Height: ')
        custom_height_hbox.pack_start(self.custom_height_label)
        self.custom_height_entry = gtk.Entry()
        self.custom_height_entry.set_width_chars(5)
        custom_height_hbox.pack_start(self.custom_height_entry, False, False, 3)
        custom_vbox.pack_start(custom_height_hbox, True, True, 3)
        self.custom_label = gtk.Label('Custom')
        custom_vbox.pack_start(self.custom_label)
        self.custom_radio = gtk.RadioButton(self.actual_radio)
        self.custom_radio.connect("toggled", self.on_custom_radio_toggled)
        cusbox = gtk.HBox()
        cusbox.pack_start(gtk.Label())
        cusbox.pack_start(self.custom_radio)
        custom_vbox.pack_start(cusbox)

        self.on_scale_radio_toggled(self.scale_radio)
        self.on_custom_radio_toggled(self.custom_radio)

        hbox4 = gtk.HBox(False, 8)
        hbox4.pack_start(actual_vbox)
        hbox4.pack_start(gtk.VSeparator(), False, False, 0)
        hbox4.pack_start(scale_vbox)
        hbox4.pack_start(gtk.VSeparator(), False, False, 0)
        hbox4.pack_start(custom_vbox)
        self.vbox.pack_start(hbox4)

        self.show_all()

    def on_scale_radio_toggled(self, widget):
        self.scale_width_label.set_sensitive(widget.get_active())
        self.scale_width_entry.set_sensitive(widget.get_active())
        self.scale_label.set_sensitive(widget.get_active())
        if widget.get_active():
            self.scale_width_entry.grab_focus()

    def on_custom_radio_toggled(self, widget):
        self.custom_width_label.set_sensitive(widget.get_active())
        self.custom_width_entry.set_sensitive(widget.get_active())
        self.custom_height_label.set_sensitive(widget.get_active())
        self.custom_height_entry.set_sensitive(widget.get_active())
        self.custom_label.set_sensitive(widget.get_active())
        if widget.get_active():
            self.custom_width_entry.grab_focus()

    def on_actual_radio_toggled(self, widget):
        self.actual_label.set_sensitive(widget.get_active())

    def on_browse_button_clicked(self, widget):
        response = self.file_dialog.run()
        if response == gtk.RESPONSE_OK:
            self.set_location(self.file_dialog.get_filename())
        self.file_dialog.hide()

    def set_location(self, text):
        self.entry.set_text(text)

    def get_location(self):
        return self.entry.get_text()

    def get_position(self):
        if self.fleft_radio.get_active():
            return "float-left"
        elif self.fright_radio.get_active():
            return "float-right"
        else:
            return "center"

    def validate_size(self):
        if self.scale_radio.get_active():
            width = self.scale_width_entry.get_text()
            if not len(width):
                return False
            for char in width:
                if not char.isdigit():
                    return False
        elif self.custom_radio.get_active():
            width = self.custom_width_entry.get_text()
            if not len(width):
                return False
            for char in width:
                if not char.isdigit():
                    return False
            height = self.custom_height_entry.get_text()
            if not len(height):
                return False
            for char in height:
                if not char.isdigit():
                    return False
        return True

    def get_size(self):
        if self.actual_radio.get_active():
            return ''
        elif self.scale_radio.get_active():
            return "?%s" % self.scale_width_entry.get_text()
        else:
            return "?%sx%s" % (self.custom_width_entry.get_text(), self.custom_height_entry.get_text())



class YesNo(Hig):
    def __init__(self, parent, pritext, sectext, prefix=None):
        """HIG compliant Yes/No dialog"""
        Hig.__init__(
                self, parent, pritext, sectext, gtk.STOCK_DIALOG_QUESTION,
                [ [ gtk.STOCK_NO, gtk.RESPONSE_NO ],
                [gtk.STOCK_YES, gtk.RESPONSE_YES] ], gtk.RESPONSE_YES
        )

class Error(Hig):
    def __init__(self, parent, pritext, sectext):
        """HIG compliant error dialog."""
        Hig.__init__(
            self, parent, pritext, sectext, gtk.STOCK_DIALOG_ERROR,
            [ [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ]
        )

class FileOverwrite(Hig):
    def __init__(self, parent, file):
        """Overwrite a file dialog."""
        Hig.__init__(
            self, parent, "Overwrite existing file?",
            "The file '" + file + "' already exists. If you choose to overwrite the file, its contents will be lost.", gtk.STOCK_DIALOG_WARNING,
            [ [ gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL ], [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ],
            gtk.RESPONSE_CANCEL
        )

    def run(self):
        return Hig.run(self) == gtk.RESPONSE_OK


class Confirm(Hig):
    def __init__(self, parent, pritext, sectext):
        """HIG compliant confirmation dialog."""
        Hig.__init__(
            self, parent, pritext, sectext, gtk.STOCK_DIALOG_QUESTION,
            [ [gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL], [ gtk.STOCK_OK, gtk.RESPONSE_OK ] ]
        )

class AboutDialog(NewtonGui):
    def __init__(self):
        NewtonGui.__init__(self)
        self.dialog = self.wt.get_widget('about_dialog')
        self.dialog.set_icon(self.icon)
        self.dialog.set_logo(self.icon)
        self.dialog.set_version(VERSION)
        gtk.about_dialog_set_email_hook(self._url_hook, "mailto:")
        gtk.about_dialog_set_url_hook(self._url_hook, "")

    def run(self, uicomponent=None, verb=None):
        self.dialog.run()
        self.dialog.destroy()

    def _email_hook(self, widget, url):
        gnomevfs.url_show('mailto:' + url)

    def _url_hook(self, widget, url, scheme):
        gnome.ui.url_show_on_screen(scheme + url, widget.get_screen())

gobject.signal_new("spellcheck-bug", EditWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-location-changed", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-link-clicked", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-edit-clicked", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-link-message", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-save-clicked", EditWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-new-clicked", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-delete-clicked", BrowserWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
gobject.signal_new("wiki-cancel-clicked", EditWidget, gobject.SIGNAL_ACTION, gobject.TYPE_STRING, (gobject.TYPE_STRING, ))
