# Copyright (c) 2004 Dennis Craven
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

from newton import VERSION, WIKI_STYLES, PIXMAPS_DIR
from gettext import gettext as _

WIKI_HOME_TEXT = '@@@ Never' + \
_(''' 
This is your NewtonHome page. You may edit this page however you like, but you can **not** delete it. This page will be loaded each time [[http://newton.sf.net|Newton]] is started.

[[http://newton.sf.net|Newton]] is a serverless wiki applet for the GNOME2 Desktop Environment. It is meant to be easy to use, with little or no setup.

Please remember that [[http://newton.sf.net|Newton]] is in its early stages of development. Bugs and feature requests should be sent to the [[newton-users@lists.sourceforge.net|newton-users]] mailing list.

To add pages, add links to an existing page, then click on those links. Alternatively, click the //New// button.

== Features Included with this Version

* Edit and view attractive wiki pages.
* Simple editing with an edit toolbar and easy to use dialogs.
* Rendering power of the Gecko (used in [[http://www.firefox.com|Firefox]]) embedded in the viewing window.
* Embed links to media files (text, mp3, pdf, etc) and have them open in your default viewer when clicked.
* Email links open in your default email client and Internet links open in your default browser.
* A simple, and mostly complete wiki syntax for formatting text.
** Headings/Section
** **Bold**, __underlined__, //italic//, <del>strikethrough</del>, and ''monospaced'' fonts.
** Embed links (URLs) into your documents.
** Embed remote or local images into your documents with variable size and position.
** Code blocks.
** Ordered and unordered lists.
** Other small things like horizontal rules etc...
* A wiki engine to translate wiki text to XHTML.
* GNOME2 applet for quick access to all of your favourite wiki documents.
* Tree view pane for easy navigation between wiki pages.
* Preferences dialog and actions
** Ability to ignore CamelCase links
** Switch between wiki styles (apparently some people don't like brown)
* Two available styles; blue and brown (using Cascading Stylesheets)
''')

WIKI_SYNTAX_TEXT = '@@@ Never' + \
_(""" 
= Document Formatting Syntax

[[http://newton.sf.net|Newton]] supports many types of document formatting through the use of a simple markup language. The markup language is easy to learn so that you can start formatting your documents very quickly. This page contains a description of all formatting supported by [[http://newton.sf.net|Newton]]. Each description will consist of both the wiki syntax needed for that formatting, and the formatted result.

== Basic Text Formatting

[[http://newton.sf.net|Newton]] supports **bold**, ''monospaced'', //italic//, <del>strikethrough</del>, and __underlined__ text. These can also be done in **//__combination__//**!

  Newton supports **bold**, ''monospaced'', //italic//, &lt;del&gt;strikethrough&lt;/del&gt;, and __underlined__ text. These can also be done in **//__combination__//**!

You can also make <sub>subscripts</sub> and <sup>superscripts</sup>!

  You can also make &lt;sub&gt;subscripts&lt;/sub&gt; and &lt;sup&gt;superscripts&lt;/sup&gt;!

== Unordered and Ordered Lists

Both ordered (//ie. numbered//) and unordered (//ie. bulleted//) lists are supported. Start a line with a ''#'' to add a ordered item and a ''*'' to add an unordered item. The number of ''#'''s or ''*'''s starting the line determines the level of the item in the list. For example,

  * First level unordered item 1
  * First level unordered item 2
  ** This is a second level item
  * First level item 3

Produces this output:

* First level unordered item 1
* First level unordered item 2
** This is a second level item
* First level item 3

Similarly, for an ordered list,

  # First level ordered item 1
  # First level ordered item 2
  ## This is a second level item
  # First level item 3

Produces this output:

# First level ordered item 1
# First level ordered item 2
## This is a second level item
# First level item 3

== Headings or Sections

[[http://newton.sf.net|Newton]] supports up to five levels of headings. Start a line with ''='' for a first level heading, ''=='' for a second level heading, ''==='' for a third level, and so on.

  = Heading 1
  Some text.
  == Heading 2
  Some more text.
  === Heading 3
  And so on...

= Heading 1
Some text.
== Heading 2
Some more text.
=== Heading 3
And so on...

== Internal and External Links

Links can be created to other [[http://newton.sf.net|Newton]] pages or to webpages on the Internet. Internal links can be created by simply typing a word in "camelcase" format. Like this, NewtonHome. If the wiki page that the link points to exists, it will appear green like the case of NewtonHome. Otherwise it will appear red like NonExistantPage.

You can make external links like [[http://www.pygtk.org|this]] or like this ([[http://www.pygtk.org]]).

  You can make external links like [[http://www.pygtk.org|this]] or like this ([[http://www.pygtk.org]]).

Email is simple too! An email link is as easy as this [[dcraven@gmail.com|dcraven]] or this [[dcraven@gmail.com]].

  Email is simple too! An email link is as easy as this [[dcraven@whatever.com|dcraven]] or this [[dcraven@whatever.com]].

== Embedded Images

{{ http://www.google.ca/intl/en_ca/images/logo.gif}}
Both local images and remote (on the Internet) images can be embedded into your documents easily with a simple syntax. There is also a GUI dialog available in the edit screen to make it even easier!

For example, if you are connected to the Internet right now, you should see the Google logo to the right of this text.

To embed this image, I used the following syntax:
  {{ http://www.google.ca/intl/en_ca/images/logo.gif}}

Notice the space to the left of the URL where there is none on the right side in the above example? That means that we want the image to "float right". With this syntax, the image will appear on the right side of the document while the text neatly wraps around it. Think of it like this; there is a space on the left, because the image has been //shoved// to the right. The same works to make the image "float left" by simply putting the space on the right.

If you want the image to appear centered in your document, put a space on both sides like this:
  {{ http://www.google.ca/intl/en_ca/images/logo.gif }}

And now the image appears centered. {{ http://www.google.ca/intl/en_ca/images/logo.gif?200 }}

That image appears smaller though because I also used the simple image scaling syntax. The syntax that I used for the centered image looks like this:
  {{ http://www.google.ca/intl/en_ca/images/logo.gif?200 }}

Notice the "''?200''" part after the URL? This means that you want the image to appear 200 pixels wide, and scale the height automatically so that the image doesn't look stretched. If you **do** want to //stretch// the image, you can specify both width **and** height like this:

{{ http://www.google.ca/intl/en_ca/images/logo.gif?500x50 }}
  {{ http://www.google.ca/intl/en_ca/images/logo.gif?500x50 }}

That means you want the image to be scaled to 500 pixels wide by 50 pixels high.

Locally stored (images on your own computer) can be embedded as well. Just specify the local path to the image file instead of a URL. For example, I could enter:

  {{ /home/dcraven/newton.png }}

And that would display the image called ''newton.png'', centered in my document. The same scaling and positioning syntax that applied for remote images still applies exactly the same way for local images.

== Embedding Other Media Files

[[http://newton.sf.net|Newton]] can also embed links to other types of media files such as mp3, pdf, word processor documents, or even plain text documents. When you click on the embedded link, the document will be opened using your default viewer.

To embed a PDF file called ''myfile.pdf'', simply put the file's path between sets of double curly braces. Just like we did with images. For example,

{{/home/dcraven/myfile.pdf}}
  {{/home/dcraven/myfile.pdf}}

If the file does not exist in the location specified, it will be red (like the one above) and [[http://newton.sf.net|Newton]] will tell you the file doesn't exist when you click on it. Otherwise it will appear in grey. You can also give the link a name in case you don't want the whole file path to appear in your document. 

{{/home/dcraven/myfile.pdf|My PDF File}}
  {{/home/dcraven/myfile.pdf|My PDF File}}

== Miscellaneous Formatting

* New paragraphs are created by adding a blank line.
* Horozontal rules or lines are made by putting four or more dashes on a line by themselves. Like this,
  ----

----
* Code blocks (sections with wiki markup ignored, like the examples above) are created by starting a line with two spaces.
""")

WIKI_STYLE_SHEET = \
"""
body {
  font: 80% "Bitstream Vera Sans", "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
  background-color: White;
  color: Black;
  margin: 2px;
  padding: 2px;
}

div.clearer {clear: both; line-height: 0; height: 0;}

.pagename {
	height:25px;
	font-size:200%;
	font-weight:bolder;
	color:#d19972;
	text-align:left;
	padding:0 0 0.5cm 0;
    padding-top: 0.25cm;
}

span.pagename {
	height:25px;
	font-size:100%;
	font-weight:bolder;
	text-align:left;
	padding:0 0 0.5cm 0;
    padding-top: 0.25cm;
    color: #767676;
}

.pagename a:link, .pagename a:visited {
  color:#436976;
  text-decoration:none;
}

.pagename a:hover, .pagename a:active {
  color:#FF9933;
}

.bar {
	height:15px;
	border-top:2px solid #a36d47;
	border-bottom:1px solid #a36d47;
	background:#d19972;
	padding:2px;
	margin:0;
}

table {
  font-size: 100%;
  padding:0;
  margin:0;
}

tr,td,th {padding:0; margin:0;}

img {border:0}

p {padding:0; margin: 1.0em 0 1.0em 0;}

hr {
  border: 0px;
  border-top: 1px solid #a36d47;
  text-align:center;
  height: 0px;
}

span.search {
    background: #3DFE40;
}

a {
  color:#436976;
  text-decoration:none;
}
a:hover {
  color:#000000;
  text-decoration:underline;
}

div.breadcrumbs{
  background-color: #fff4b5;
  text-align: right;
  font-size:80%;
  color: #666666;
  padding-left: 4px;
}

a.urlextern{
    background: transparent url(@PIXMAPS_DIR@/link_icon.gif) 0px 1px no-repeat;
    padding: 1px 0px 1px 16px;
    color:#436976;
    text-decoration:none;
}

a.urlextern:visited {
  color:Purple;
}
a.urlextern:hover {
  text-decoration:underline;
}

a.interwiki{
    background: transparent url(@PIXMAPS_DIR@/interwiki.png) 0px 1px no-repeat;
    padding: 1px 0px 1px 16px;
    color:#436976;
    text-decoration:none;
}
a.interwiki:visited {
  color:Purple;
}
a.interwiki:hover {
  text-decoration:underline;
}

a.media {
  color:#436976;
  text-decoration:none;
}
a.media:hover {
  color:#436976;
  text-decoration:underline
}

img.linkicon {
	vertical-align: text-bottom;
}

img.media {
  margin: 3px;
}

img.medialeft {
  margin: 0 1.5em 0 0;
  border: 0;
  float: left;
}

img.mediaright {
  margin: 0 0 0 1.5em;
  border: 0;
  float: right;
}

a.mail {
  background: transparent url(@PIXMAPS_DIR@/mail_icon.gif) 0px 1px no-repeat;
  padding: 1px 0px 1px 16px;
  color:#436976;
  text-decoration:none;
}
a.mail:hover {
  text-decoration:underline;
}

a.wikilink1:link    { color:#009900; text-decoration:none }
a.wikilink1:visited { color:#009900; text-decoration:none }
a.wikilink1:hover   { color:#009900; text-decoration:underline }

a.wikilink2:link    { color:#FF3300; text-decoration:none }
a.wikilink2:visited { color:#FF3300; text-decoration:none }
a.wikilink2:hover   { color:#FF3300; text-decoration:underline }

a.wikilink3:link    { color:#436976; text-decoration:none }
a.wikilink3:visited { color:#436976; text-decoration:none }
a.wikilink3:hover   { color:#436976; text-decoration:underline }

acronym {
  cursor: help;
  border-bottom: 1px dotted #000;
}

div.page {
  margin-top: 4px;
  margin-left:  1em;
  margin-right: 2em;
  text-align: justify;
}

h1 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    border-bottom: 1px solid #a36d47;
    clear: left;
    font-size: 160%; 
    margin-left: 0px; 
    font-weight: bold;
}
h2 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    font-weight: normal;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    border-bottom: 1px solid #a36d47;
    clear: left;
    font-size: 150%; 
    margin-left: 20px;
}
h3 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 140%; 
    margin-left: 40px; 
    border-bottom: none; 
    font-weight: bold;
}
h4 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 120%; 
    margin-left: 60px; 
    border-bottom: none; 
    font-weight: bold;
}
h5 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 100%; 
    margin-left: 80px; 
    border-bottom: none; 
    font-weight: bold;
}

div.level1 {margin-left: 3px;}
div.level2 {margin-left: 23px;}
div.level3 {margin-left: 43px;}
div.level4 {margin-left: 63px;}
div.level5 {margin-left: 83px;}

ul { 
  line-height: 1.5em;
  list-style-type: disc;
  color: #986746;
  margin: 0.5em 0 0.5em 1.5em;
  padding: 0;
}

ol {
  line-height: 1.5em;
  margin: 0.5em 0 0.5em 1.5em;
  padding: 0;
  color: #986746;
}

span.righttext {
  float: right;
}

span.li {
  color: #000000;
  font-weight: normal;
}

li.open {
  list-style-image: url(images/open.gif);
}

li.closed {
  list-style-image: url(images/closed.gif);
}
.quote {
  border-left: 2px solid #a36d47;
  padding-left: 3px;
}

pre.pre {
  font-family: mono;
  padding: 0.5em;
  border: 1px dashed #a36d47;
  color: Black;
  background-color: #fff4b5;
  overflow: auto;
  font-weight: normal;
}

pre.code {
  font-family: mono;
  padding: 0.5em;
  border: 1px dashed #a36d47;
  color: Black;
  background-color: #fff4b5;
  overflow: auto;
  font-weight: normal;
}

code {
  font-family: mono;
}
""" 

WIKI_STYLE_BLUE = \
"""

body {
  font: 80% "Bitstream Vera Sans", "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
  background-color: White;
  color: Black;
  margin: 2px;
  padding: 2px;
}

div.clearer {clear: both; line-height: 0; height: 0;}

.pagename {
	height:25px;
	font-size:200%;
	font-weight:bolder;
	color:#62618D;
	text-align:left;
	padding:0 0 0.5cm 0;
    padding-top: 0.25cm;
}

span.pagename {
	height:25px;
	font-size:100%;
	font-weight:bolder;
	text-align:left;
	padding:0 0 0.5cm 0;
    padding-top: 0.25cm;
    color: #767676;
}

.pagename a:link, .pagename a:visited {
  color:#436976;
  text-decoration:none;
}

.pagename a:hover, .pagename a:active {
  color:#FF9933;
}

.bar {
	height:15px;
	border-top:2px solid #1A1786;
	border-bottom:1px solid #1A1786;
	background:#62618D;
	padding:2px;
	margin:0;
}

table {
  font-size: 100%;
  padding:0;
  margin:0;
}

tr,td,th {padding:0; margin:0;}

img {border:0}

p {padding:0; margin: 1.0em 0 1.0em 0;}

hr {
  border: 0px;
  border-top: 1px solid #a36d47;
  text-align:center;
  height: 0px;
}

span.search {
    background: #3DFE40;
}

a {
  color:#436976;
  text-decoration:none;
}
a:hover {
  color:#000000;
  text-decoration:underline;
}

div.breadcrumbs{
  background-color: #ABA9F4;
  text-align: right;
  font-size:80%;
  color: #666666;
  padding-left: 4px;
}

a.urlextern{
    background: transparent url(@PIXMAPS_DIR@/link_icon.gif) 0px 1px no-repeat;
    padding: 1px 0px 1px 16px;
    color:#436976;
    text-decoration:none;
}

a.urlextern:visited {
  color:Purple;
}
a.urlextern:hover {
  text-decoration:underline;
}

a.interwiki{
    background: transparent url(@PIXMAPS_DIR/interwiki.png) 0px 1px no-repeat;
    padding: 1px 0px 1px 16px;
    color:#436976;
    text-decoration:none;
}
a.interwiki:visited {
  color:Purple;
}
a.interwiki:hover {
  text-decoration:underline;
}

a.media {
  color:#436976;
  text-decoration:none;
}
a.media:hover {
  color:#436976;
  text-decoration:underline
}

img.linkicon {
	vertical-align: text-bottom;
}

img.media {
  margin: 3px;
}

img.medialeft {
  margin: 0 1.5em 0 0;
  border: 0;
  float: left;
}

img.mediaright {
  margin: 0 0 0 1.5em;
  border: 0;
  float: right;
}

img.mediaright {
  margin: 5px;
  border: 0;
  float: right;
}

a.mail {
  background: transparent url(@PIXMAPS_DIR@/mail_icon.gif) 0px 1px no-repeat;
  padding: 1px 0px 1px 16px;
  color:#436976;
  text-decoration:none;
}
a.mail:hover {
  text-decoration:underline;
}

a.wikilink1:link    { color:#009900; text-decoration:none }
a.wikilink1:visited { color:#009900; text-decoration:none }
a.wikilink1:hover   { color:#009900; text-decoration:underline }

a.wikilink2:link    { color:#FF3300; text-decoration:none }
a.wikilink2:visited { color:#FF3300; text-decoration:none }
a.wikilink2:hover   { color:#FF3300; text-decoration:underline }

a.wikilink3:link    { color:#436976; text-decoration:none }
a.wikilink3:visited { color:#436976; text-decoration:none }
a.wikilink3:hover   { color:#436976; text-decoration:underline }

acronym {
  cursor: help;
  border-bottom: 1px dotted #000;
}

div.page {
  margin-top: 4px;
  margin-left:  1em;
  margin-right: 2em;
  text-align: justify;
}

h1 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    border-bottom: 1px solid #62618D;
    clear: left;
    font-size: 160%; 
    margin-left: 0px; 
    font-weight: bold;
}
h2 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    font-weight: normal;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    border-bottom: 1px solid #62618D;
    clear: left;
    font-size: 150%; 
    margin-left: 20px;
}
h3 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 140%; 
    margin-left: 40px; 
    border-bottom: none; 
    font-weight: bold;
}
h4 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 120%; 
    margin-left: 60px; 
    border-bottom: none; 
    font-weight: bold;
}
h5 {
    color: Black;
    background-color: transparent;
    font-family: "Lucida Grande", Verdana, Lucida, Helvetica, Arial, sans-serif;
    margin-right:   0;
    margin-top:     0;
    margin-bottom:  1em;
    padding-left:   0;
    padding-right:  0;
    padding-top:    0.5em;
    padding-bottom: 0;
    clear: left;
    font-size: 100%; 
    margin-left: 80px; 
    border-bottom: none; 
    font-weight: bold;
}

div.level1 {margin-left: 3px;}
div.level2 {margin-left: 23px;}
div.level3 {margin-left: 43px;}
div.level4 {margin-left: 63px;}
div.level5 {margin-left: 83px;}

ul { 
  line-height: 1.5em;
  list-style-type: disc;
  color: #62618D;
  margin: 0.5em 0 0.5em 1.5em;
  padding: 0;
}

ol {
  line-height: 1.0em;
  margin: 0.5em 0 0.5em 1.5em;
  padding: 0;
  color: #62618D;
  font-weight: bold;
}

span.righttext {
  float: right;
}

span.li {
  color: #000000;
  font-weight: normal;
}

li.open {
  list-style-image: url(images/open.gif);
}

li.closed {
  list-style-image: url(images/closed.gif);
}
.quote {
  border-left: 2px solid #a36d47;
  padding-left: 3px;
}

pre.pre {
  font-family: mono;
  padding: 0.5em;
  border: 1px dashed #a36d47;
  color: Black;
  background-color: #fff4b5;
  overflow: auto;
  font-weight: normal;
}

pre.code {
  font-family: mono;
  padding: 0.5em;
  border: 1px dashed #a36d47;
  color: Black;
  background-color: #fff4b5;
  overflow: auto;
  font-weight: normal;
}

code {
  font-family: mono;
  font-weight: normal;
}
"""

WIKI_PAGE_HEADER = \
'''
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
    <head>
        <title>@PAGENAME@</title>
        <link rel="stylesheet" media="screen" type="text/css" href="@STYLESHEET@" />
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
        <meta name="generator" content="Newton %s " />
        <style type="text/css">@STYLE@</style>

    </head>
    <body>
        <div class="all">
            <div class="stylehead">
                <div class="header">
                    <div class="pagename">
                        [[<span class="pagename">@PAGENAME@</span>]]
                    </div>
                </div>

                <div class="bar" id="bar_top">
                </div>
                <div class="breadcrumbs">
                    %s: @MODDATE@
                </div>
            </div>
'''  % (VERSION, _('Last modified'))

WIKI_PAGE_FOOTER = \
'''
        <div class="clearer">&nbsp;</div>
        <div class="stylefoot">
            <div class="bar" id="bar_bottom">
                <div class="bar-left" id="bar_bottomleft">
                </div>
                <div class="bar-right" id="bar_bottomright">
                </div>
            </div>
            <div align="center" class="footerinc">
            </div>
        </div>
    </div>
    </body>
</html>
'''

